import { HtmlUtil } from "./htmlUtil.js";
import { moduleName } from "./settings.js";
import { createPicturesContent } from "./viewTemplates/viewPicture.js";
import { createReferenseContentTabs } from "./viewTemplates/viewReferense.js";
import { createAwardsTab } from "./viewTemplates/viewAwards.js";
import { createLinksTab } from "./viewTemplates/viewLinks.js";
import { createColorContentTab } from "./viewTemplates/viewColor.js";


export class DataBook{
    fileImg;
    filePath;
    buttonImg;
    rounded;
    userColors = {};
}

let dataBook = new DataBook();

async function refreshCompendium(actorsSelect, itemsSelect, rollTableSelect, journalEntrySelect, macroSelect, sceneSelect){
    dataBook.actorsSelect = {};
    dataBook.itemsSelect = {};
    dataBook.rollTableSelect = {};
    dataBook.journalEntrySelect = {};
    dataBook.macroSelect = {};
    dataBook.sceneSelect = {};
    const compendiums = game.packs.map(pack => {
        return {
        name: pack.metadata.label,
        collection: pack.collection,
        entities: pack.index.size
        };
    });
    actorsSelect.empty();
    itemsSelect.empty();
    rollTableSelect.empty();
    journalEntrySelect.empty();
    macroSelect.empty();
    sceneSelect.empty();
    for (const compendium of compendiums) {
        const pack = game.packs.get(compendium.collection);
        if (pack.metadata.type === "Actor") {  
            let optGoup = document.createElement('option');
            optGoup.setAttribute('label', compendium.name + ' [' + compendium.collection.split('.')[0] + ']');
            optGoup.setAttribute('value', compendium.collection);
            dataBook.actorsSelect[compendium.collection] = {};
            for (let actor of pack.index){
                dataBook.actorsSelect[compendium.collection][`${actor.name} [${compendium.collection}]`] = [`[Compendium.${pack.metadata.id}.${pack.metadata.type}.${actor._id}]`, actor.img];
            }
            actorsSelect.append(optGoup);
        }
        if (pack.metadata.type === "Item") { 
            let optGoup = document.createElement('option');
            optGoup.setAttribute('label', compendium.name + ' [' + compendium.collection.split('.')[0] + ']');
            optGoup.setAttribute('value', compendium.collection);
            dataBook.itemsSelect[compendium.collection] = {};
            for (let item of pack.index){
                dataBook.itemsSelect[compendium.collection][`${item.name} [${compendium.collection}]`] = [`[Compendium.${pack.metadata.id}.${pack.metadata.type}.${item._id}]`, item.img];
            }
            itemsSelect.append(optGoup);
        }
        if (pack.metadata.type === "RollTable") {  
            let optGoup = document.createElement('option');
            optGoup.setAttribute('label', compendium.name + ' [' + compendium.collection.split('.')[0] + ']');
            optGoup.setAttribute('value', compendium.collection);
            dataBook.rollTableSelect[compendium.collection] = {};
            for (let actor of pack.index){
                dataBook.rollTableSelect[compendium.collection][`${actor.name} [${compendium.collection}]`] = [`[Compendium.${pack.metadata.id}.${pack.metadata.type}.${actor._id}]`, actor.img];
            }
            rollTableSelect.append(optGoup);
        }
        if (pack.metadata.type === "JournalEntry") { 
            let optGoup = document.createElement('option');
            optGoup.setAttribute('label', compendium.name + ' [' + compendium.collection.split('.')[0] + ']');
            optGoup.setAttribute('value', compendium.collection);
            dataBook.journalEntrySelect[compendium.collection] = {};
            for (let item of pack.index){
                dataBook.journalEntrySelect[compendium.collection][`${item.name} [${compendium.collection}]`] = [`[Compendium.${pack.metadata.id}.${pack.metadata.type}.${item._id}]`, item.img];
            }
            journalEntrySelect.append(optGoup);
        }
        if (pack.metadata.type === "Macro") {  
            let optGoup = document.createElement('option');
            optGoup.setAttribute('label', compendium.name + ' [' + compendium.collection.split('.')[0] + ']');
            optGoup.setAttribute('value', compendium.collection);
            dataBook.macroSelect[compendium.collection] = {};
            for (let actor of pack.index){
                dataBook.macroSelect[compendium.collection][`${actor.name} [${compendium.collection}]`] = [`[Compendium.${pack.metadata.id}.${pack.metadata.type}.${actor._id}]`, actor.img];
            }
            macroSelect.append(optGoup);
        }
        if (pack.metadata.type === "Scene") { 
            let optGoup = document.createElement('option');
            optGoup.setAttribute('label', compendium.name + ' [' + compendium.collection.split('.')[0] + ']');
            optGoup.setAttribute('value', compendium.collection);
            dataBook.sceneSelect[compendium.collection] = {};
            for (let item of pack.index){
                dataBook.sceneSelect[compendium.collection][`${item.name} [${compendium.collection}]`] = [`[Compendium.${pack.metadata.id}.${pack.metadata.type}.${item._id}]`, item.img];
            }
            sceneSelect.append(optGoup);
        }
    }
}


export class BookOfMagicInk extends FormApplication {

    static get defaultOptions() {
        return foundry.utils.mergeObject(super.defaultOptions, {
            object: {},
            template: "./modules/arcane-ink/templates/book.hbs",
            popOut: true,
            resizable: true,
            title: game.i18n.localize("MI.name"),
            tabs: [
                {navSelector: ".tabs", contentSelector: ".content", initial: "tab1"}, 
                {navSelector: ".referenseTabs", contentSelector: ".contentRef", initial: "tab1"}
            ]
        });
    }

    static get defaultSettings() {
        return {
            hotkey: "KeyJ", // Клавиша по умолчанию для открытия окна
        };
    }

    static registerSettings() {
        game.settings.register("arcane-ink", "hotkey", {
            name: "Hotkey for Module",
            hint: "Select the hotkey to open the module's window.",
            scope: "world",
            config: true,
            type: String,
            default: BookOfMagicInk.defaultSettings.hotkey,
            onChange: () => this.updateHotkey(), // Обновляем горячую клавишу при изменении
        });
    }

    static updateHotkey() {
        const hotkey = game.settings.get("arcane-ink", "hotkey");
        this.bindHotkey(hotkey);
    }

    static bindHotkey(hotkey) {
        // Удаляем предыдущий слушатель, если есть
        if (this.currentHotkey) {
            window.removeEventListener("keydown", this.handleKey);
        }

        this.currentHotkey = hotkey;
        window.addEventListener("keydown", this.handleKey.bind(this));
    }

    static handleKey(event) {
        if (event.code === this.currentHotkey) {
            event.preventDefault(); // Предотвращаем стандартное поведение клавиши
            this.openModuleWindow(); // Вызов окна модуля
        }
    }

    static openModuleWindow() {
        if (game.user.isGM){
            HtmlUtil.book.render(true);
        }
        console.log("Открываем окно модуля! 🚀");
    }

    getSelectedText(){
        let selectedText = document.getSelection().getRangeAt(0).cloneContents();
        const serializer = new XMLSerializer();
        selectedText = serializer.serializeToString(selectedText);
        selectedText = selectedText.replaceAll('xmlns="http://www.w3.org/1999/xhtml"', "");
        return selectedText
    }

    async pathPicker(inputForm, img, butImg){
        let path = new FilePicker();
        dataBook.filePath = inputForm;
        dataBook.fileImg = img;
        dataBook.buttonImg = butImg;
        path.callback = this.make_path;
        path.render();
    }

    async make_path(args){
        game.user.setFlag(moduleName, "picturePath", args);
        dataBook.filePath[0].value = args;
        dataBook.fileImg[0].src = args;
        dataBook.buttonImg[0].src = args;
    }

    getData(app, html) {
        const data = super.getData();

        data.footer = `${game.i18n.localize("MI.footer")}<a href="${game.i18n.localize("MI.boostSite")}">${game.i18n.localize("MI.boostSite")}</a>`;

        data.tabs = [{ 
            id: "tab1",
            label: "tab1",
            title: game.i18n.localize("MI.mainTabs.pictureAndDesign"),
            content: createPicturesContent()
        },
        { 
            id: "tab2",
            label: "tab2",
            title: game.i18n.localize("MI.mainTabs.reference"),
            content: createReferenseContentTabs()
        },
        { 
            id: "tab3",
            label: "tab3",
            title: game.i18n.localize("MI.mainTabs.links"),
            content: createLinksTab()
        },
        { 
            id: "tab4",
            label: "tab4",
            title: game.i18n.localize("MI.mainTabs.award"),
            content: createAwardsTab()
        },
        { 
            id: "tab5",
            label: "tab5",
            title: game.i18n.localize("MI.mainTabs.color"),
            content: createColorContentTab()
        },
    ];
        return data;
    }

    async formLoading(html){
        dataBook.color = '#ffffff';
        dataBook.backgroundColorBorder = 'rgba(0, 0, 0, 0)';
        dataBook.colorHex = html.find('[id="colorHex"]');
        dataBook.colorViget = html.find('[id="colour"]');
        dataBook.select = html.find('[id="userPalette"]');
        dataBook.filePath = html.find('[name="picturesInput"]');
        dataBook.fileImg = html.find('[name="refImg"]');
        dataBook.buttonImg = html.find('[id="butImg"]');
        dataBook.filePath[0].value = (game.user.getFlag(moduleName, "picturePath") !== undefined) ? game.user.getFlag(moduleName, "picturePath") : "icons/vtt-512.png";
        dataBook.fileImg[0].src = dataBook.filePath[0].value;
        dataBook.buttonImg[0].src = dataBook.filePath[0].value;
        dataBook.userColors = game.user.getFlag(moduleName, "userColors") ? game.user.getFlag(moduleName, "userColors"): {};
        if (game.user.getFlag(moduleName, "rounded")){
            let round = html.find('[name="onCircle"]');
            await round.prop( "checked", true);
            await round.trigger('change');
        }
        dataBook.select.find('option').remove();
        for (const [_color, _name] of Object.entries(dataBook.userColors)) {
            await dataBook.select.append(`<option style="background-color: ${_color};" value=${_color} >${_name}</option>`);
        }
        await dataBook.select.trigger('change');
    }
    activateListeners(html) {
        this.formLoading(html);

        html.on('click', '#refImg', () => {
            this.pathPicker(html.find('[name="picturesInput"]'), html.find('[name="refImg"]'), html.find('[id="butImg"]'));
        })

        html.on('change', '#picturesInput', () => {
            this.make_path(html.find('[name="picturesInput"]')[0].value);
        })  

        html.on('click', '#addPictures', () => {
            let scale = html.find('[name="scale"]')[0].value;
            let direction = '';
            let circle = '';
            if (html.find('[name="leftCheck"]')[0].checked) {
                direction = 'left';
            }
            if (html.find('[name="onCircle"]')[0].checked) {
                circle = 'class="round"';
            }
            if (html.find('[name="rightCheck"]')[0].checked) {
                direction = 'right';
            }
            let width = html.find('[name="widthPic"]')[0].value;
            let height = html.find('[name="heightPic"]')[0].value;
            let icon = html.find('[name="picturesInput"]')[0].value;
            let selectedText = document.getSelection().toString();
            let newContent = `<figure class="${direction} ${scale}"><img src=${icon} ${circle} width="${width}" height="${height}"><figcaption><p>${selectedText}</p></figcaption></figure>`;
            document.execCommand('insertHTML', false, newContent);
          })

        html.on('click', '#buttonAbility', () => {
            let num = new RegExp('\\d{1,3}');
            let selectedText = document.getSelection().toString();
            let check = "/check";
            let passive = '';
            let result;
            if (html.find('[id="skillPassiveCheck"]')[0].checked){
                passive = 'passive ';
            }
            let modify = html.find('[id="numberForMod"]')[0].value | 0;
            let ability = html.find('[name="ability"]')[0].value;
            if (html.find('[name="abilityCheck"]')[0].checked) {
                check = '/save';
            }
            if (html.find('[id="numberCheck"]')[0].checked){
                result = selectedText.match(num);
                if (result){
                    modify = result[0];
                }
            }
            let newContent = `[[${check} ${passive}${ability} ${modify}]]{${selectedText}}`;
            if (html.find('[id="NullCheck"]')[0].checked | !selectedText){
                newContent = `[[${check} ${passive}${ability} ${modify}]]`;
            }
            document.execCommand('insertHTML', false, newContent);
          })

        html.on('click', '#buttonSkill', () => {
            let pattern = new RegExp('[\\d]{1,2}[d,D,к,К][\\d]{1,3}');
            let num = new RegExp('\\d{1,3}');
            let selectedText = document.getSelection().toString();
            let passive = '';
            let result;
            if (html.find('[id="skillPassiveCheck"]')[0].checked){
                passive = 'passive ';
            }
            let modify = html.find('[id="numberForMod"]')[0].value | 0;
            let skill = html.find('[name="skill"]')[0].value;
            if (html.find('[id="numberCheck"]')[0].checked){
                result = selectedText.match(num);
                if (result){
                    modify = result[0];
                }
            }
            let newContent = `[[/skill ${passive}${skill} ${modify}]]{${selectedText}}`;
            if (html.find('[id="NullCheck"]')[0].checked | !selectedText){
                newContent = `[[/skill ${passive}${skill} ${modify}]]`;
            }
            document.execCommand('insertHTML', false, newContent);
        })

        html.on('click', '#buttonTypeDamage', () => {  //////22222222222222222222222222
            let pattern = new RegExp('[\\d]{1,2}[d,D,к,К,k,K,д,Д][\\d]{1,3}');
            let d = new RegExp('[D,к,К,k,K,д,Д]');
            let selectedText = document.getSelection().toString();
            let average = '';
            let damage = '/damage'
            let result;
            if (selectedText){
                result = selectedText.match(pattern);
                if (result){
                    result = result[0].replace(d, 'd')
                }
                else{
                    let num = new RegExp('\\d{1,3}');
                    result = selectedText.match(num)[0];
                }
            }
            if (html.find('[id="average"]')[0].checked) {
                average = html.find('[id="numberAverage"]')[0].value | 'true';
                if (average === 0) {
                    average = ` average`;
                }
                else{
                    average = ` average=${average}`;
                }
            }
            let damageFormula = html.find('[id="damageFormula"]')[0].value;
            damageFormula =` ${damageFormula.replace(d, 'd')}`
            let typeDamage =` ${html.find('[name="typeDamage"]')[0].value}`;
            if (typeDamage === " noType") {
                typeDamage = '';
            }else if (typeDamage === " roll"){
                damage = '/roll';
                typeDamage = '';
            }
            if (html.find('[id="numberCheck"]')[0].checked){
                if (result){
                    damageFormula =` ${result}`
                }
            }
            let newContent = `[[${damage}${damageFormula}${typeDamage}${average}]]{${selectedText}}`;
            if (html.find('[id="NullCheck"]')[0].checked | !selectedText){
                newContent = `[[${damage}${damageFormula}${typeDamage}${average}]]`;
            }
            document.execCommand('insertHTML', false, newContent);
        })

        html.on('click', '#buttonTool', () => {
            let num = new RegExp('\\d{1,3}');
            let selectedText = document.getSelection().toString();
            let passive = '';
            let result;
            if (html.find('[id="skillPassiveCheck"]')[0].checked){
                passive = ' passive';
            }
            let modify = `${html.find('[id="numberForMod"]')[0].value}` | 0;
            let tool = ` ${html.find('[name="Tool"]')[0].value}`;
            let ability = ` ${html.find('[name="abilitTooly"]')[0].value}`;
            if (html.find('[id="numberCheck"]')[0].checked){
                result = ` ${selectedText.match(num)}`;
                if (result){
                    modify = ` ${result[0]}`;
                }
            }
            console.log(result);
            let newContent = `[[/tool${passive}${tool}${ability} ${modify}]]{${selectedText}}`;
            if (html.find('[id="NullCheck"]')[0].checked | !selectedText){
                newContent = `[[/tool${passive}${tool}${ability} ${modify}]]`;
            }
            document.execCommand('insertHTML', false, newContent);
        })

        html.on('click', '#buttonCondition', () => {
            let selectedText = document.getSelection().toString();
            let condition = html.find('[name="condition"]')[0].value;
            let newContent = `&Reference[${condition}]{${selectedText}}`;
            if (html.find('[id="NullCheck"]')[0].checked | !selectedText){
                newContent = `&Reference[${condition}]`;
            }
            document.execCommand('insertHTML', false, newContent);
        })

        html.on('click', '#buttonCreatureType', () => {
            let selectedText = document.getSelection().toString();
            let creatureType = html.find('[name="creatureType"]')[0].value;
            let newContent = `&Reference[${creatureType}]{${selectedText}}`;
            if (html.find('[id="NullCheck"]')[0].checked | !selectedText){
                newContent = `&Reference[${creatureType}]`;
            }
            document.execCommand('insertHTML', false, newContent);
        })

        html.on('click', '#buttonAreaOfEffect', () => {
            let selectedText = document.getSelection().toString();
            let areaOfEffect = html.find('[name="areaOfEffect"]')[0].value;
            let newContent = `&Reference[${areaOfEffect}]{${selectedText}}`;
            if (html.find('[id="NullCheck"]')[0].checked | !selectedText){
                newContent = `&Reference[${areaOfEffect}]`;
            }
            document.execCommand('insertHTML', false, newContent);
        })

        html.on('click', '#buttonComponent', () => {
            let selectedText = document.getSelection().toString();
            let component = html.find('[name="component"]')[0].value;
            let newContent = `&Reference[${component}]{${selectedText}}`;
            if (html.find('[id="NullCheck"]')[0].checked | !selectedText){
                newContent = `&Reference[${component}]`;
            }
            document.execCommand('insertHTML', false, newContent);
        })

        html.on('click', '#buttonSpellSchool', () => {
            let selectedText = document.getSelection().toString();
            let spellSchool = html.find('[name="spellSchool"]')[0].value;
            let newContent = `&Reference[${spellSchool}]{${selectedText}}`;
            if (html.find('[id="NullCheck"]')[0].checked | !selectedText){
                newContent = `&Reference[${spellSchool}]`;
            }
            document.execCommand('insertHTML', false, newContent);
        })

        html.on('click', '#buttonRules', () => {
            let selectedText = document.getSelection().toString();
            let rulles = html.find('[name="rulesSelect"]')[0].value;
            let newContent = `&Reference[${rulles}]{${selectedText}}`;
            if (html.find('[id="NullCheck"]')[0].checked | !selectedText){
                newContent = `&Reference[${rulles}]`;
            }
            document.execCommand('insertHTML', false, newContent);
        }) 
        //refreshCompendium

        html.on('click', '#refreshCompendiumButton', () => {
            let actorsSelect = html.find('[name="actorsSelect"]');
            let itemsSelect = html.find('[name="itemsSelect"]');
            let rollTableSelect = html.find('[name="rollTableSelect"]');
            let journalEntrySelect = html.find('[name="journalEntrySelect"]');
            let macroSelect = html.find('[name="macroSelect"]');
            let sceneSelect = html.find('[name="sceneSelect"]');
            refreshCompendium(actorsSelect, itemsSelect, rollTableSelect, journalEntrySelect, macroSelect, sceneSelect)

        })

        html.on('click', '#creatureButton', () => {
            let dialogContent = HtmlUtil.createElement("div");
            dialogContent.appendChild(HtmlUtil.createElement("label", {text: `Выберите один из вариантов:\n`}));
            let selectedText;
            try {
                selectedText = this.getSelectedText();
            }catch {
                selectedText = '';
            }
            let actorsSelect = html.find('[name="actorsSelect"]').val();
            let EmbedValueLinks = html.find('[name="EmbedValueLinks"]')[0].checked;
            let checkEmptyValueLinks = html.find('[name="emptyValueLinks"]')[0].checked;
            let likeAKikimor = html.find('[name="likeAKikimor"]')[0].checked;
            let customName = html.find('[name="customName"]').val();
            let prefix = "@UUID"
            for (let actors of actorsSelect){
                for (let [actor, value] of Object.entries(dataBook.actorsSelect[actors])){
                    if (actor.toLowerCase().includes(selectedText.toLowerCase())){
                        if (EmbedValueLinks){
                            prefix = "@EMBED";
                        }
                        let result = `${prefix}${value[0]}{${selectedText}}`
                        if (customName){
                            result = `${prefix}${value[0]}{${customName}}`;
                        }
                        if (checkEmptyValueLinks){
                            result = `${prefix}${value[0]}`;
                        }
                        if (likeAKikimor){
                            result = `<section class="fvtt advice"><figure class="icon"><img src="${value[1]}" class="round"></figure><article>${result}</article></section>`
                        }
                        let button = HtmlUtil.createElement("button", {elementClass: ["creatureChangeButton"], type:"button", name: "buttonTool"});
                        let img = HtmlUtil.createElement("img", {src: value[1]});
                        let label = HtmlUtil.createElement("label", {text: actor});
                        HtmlUtil.appendElements(button, [img, label]);
                        button.setAttribute("onClick", `document.execCommand('insertHTML', false, '${result}')`)
                        dialogContent.appendChild(button);
                    }
                }
            }
            const myDialog = new Dialog({
                title: "Выбор",
                content: dialogContent.outerHTML,
                buttons: {}
            }).render(true);
        }) 

        html.on('click', '#itemButton', () => {
            let dialogContent = HtmlUtil.createElement("div");
            dialogContent.appendChild(HtmlUtil.createElement("label", {text: `Выберите один из вариантов:\n`}));
            let selectedText;
            try {
                selectedText = this.getSelectedText();
            }catch {
                selectedText = '';
            }
            let itemsSelect = html.find('[name="itemsSelect"]').val();
            let EmbedValueLinks = html.find('[name="EmbedValueLinks"]')[0].checked;
            let checkEmptyValueLinks = html.find('[name="emptyValueLinks"]')[0].checked;
            let likeAKikimor = html.find('[name="likeAKikimor"]')[0].checked;
            let customName = html.find('[name="customName"]').val();
            let prefix = "@UUID"
            for (let items of itemsSelect){
                for (let [item, value] of Object.entries(dataBook.itemsSelect[items])){
                    if (item.toLowerCase().includes(selectedText.toLowerCase())){
                        if (EmbedValueLinks){
                            prefix = "@EMBED";
                        }
                        let result = `${prefix}${value[0]}{${selectedText}}`
                        if (customName){
                            result = `${prefix}${value[0]}{${customName}}`;
                        }
                        if (checkEmptyValueLinks){
                            result = `${prefix}${value[0]}`;
                        }
                        if (likeAKikimor){
                            result = `<section class="fvtt advice"><figure class="icon"><img src="${value[1]}" class="round"></figure><article>${result}</article></section> `
                        }
                        let button = HtmlUtil.createElement("button", {elementClass: ["creatureChangeButton"], type:"button", name: "buttonTool"});
                        let img = HtmlUtil.createElement("img", {src: value[1]});
                        let label = HtmlUtil.createElement("label", {text: item});
                        HtmlUtil.appendElements(button, [img, label]);
                        button.setAttribute("onClick", `document.execCommand("insertHTML", false, '${result}')`)
                        dialogContent.appendChild(button);
                    }
                }
            }
            const myDialog = new Dialog({
                title: "Выбор",
                content: dialogContent.outerHTML,
                buttons: {}
            }).render(true);
        })

        html.on('click', '#rollTableButton', () => {
            let dialogContent = HtmlUtil.createElement("div");
            dialogContent.appendChild(HtmlUtil.createElement("label", {text: `Выберите один из вариантов:\n`}));
            let selectedText;
            try {
                selectedText = this.getSelectedText();
            }catch {
                selectedText = '';
            }
            let rollTableSelect = html.find('[name="rollTableSelect"]').val();
            let EmbedValueLinks = html.find('[name="EmbedValueLinks"]')[0].checked;
            let checkEmptyValueLinks = html.find('[name="emptyValueLinks"]')[0].checked;
            let likeAKikimor = html.find('[name="likeAKikimor"]')[0].checked;
            let customName = html.find('[name="customName"]').val();
            let prefix = "@UUID"
            for (let items of rollTableSelect){
                for (let [item, value] of Object.entries(dataBook.rollTableSelect[items])){
                    if (item.toLowerCase().includes(selectedText.toLowerCase())){
                        if (EmbedValueLinks){
                            prefix = "@EMBED";
                        }
                        let result = `${prefix}${value[0]}{${selectedText}}`
                        if (customName){
                            result = `${prefix}${value[0]}{${customName}}`;
                        }
                        if (checkEmptyValueLinks){
                            result = `${prefix}${value[0]}`;
                        }
                        if (likeAKikimor){
                            result = `<section class="fvtt advice"><figure class="icon"><img src="${value[1]}" class="round"></figure><article>${result}</article></section> `
                        }
                        let button = HtmlUtil.createElement("button", {elementClass: ["creatureChangeButton"], type:"button", name: "buttonTool"});
                        let img = HtmlUtil.createElement("img", {src: value[1]});
                        let label = HtmlUtil.createElement("label", {text: item});
                        HtmlUtil.appendElements(button, [img, label]);
                        button.setAttribute("onClick", `document.execCommand("insertHTML", false, '${result}')`)
                        dialogContent.appendChild(button);
                    }
                }
            }
            const myDialog = new Dialog({
                title: "Выбор",
                content: dialogContent.outerHTML,
                buttons: {}
            }).render(true);
        }) 

        html.on('click', '#journalEntryButton', () => { // Journal
            let dialogContent = HtmlUtil.createElement("div");
            dialogContent.appendChild(HtmlUtil.createElement("label", {text: `Выберите один из вариантов:\n`}));
            let selectedText;
            try {
                selectedText = this.getSelectedText();
            }catch {
                selectedText = '';
            }
            let journalEntrySelect = html.find('[name="journalEntrySelect"]').val();
            let EmbedValueLinks = html.find('[name="EmbedValueLinks"]')[0].checked;
            let checkEmptyValueLinks = html.find('[name="emptyValueLinks"]')[0].checked;
            let likeAKikimor = html.find('[name="likeAKikimor"]')[0].checked;
            let customName = html.find('[name="customName"]').val();
            let prefix = "@UUID"
            for (let items of journalEntrySelect){
                for (let [item, value] of Object.entries(dataBook.journalEntrySelect[items])){
                    if (item.toLowerCase().includes(selectedText.toLowerCase())){
                        if (EmbedValueLinks){
                            prefix = "@UUID"; //@EMBED
                        }
                        let result = `${prefix}${value[0]}{${selectedText}}`
                        if (customName){
                            result = `${prefix}${value[0]}{${customName}}`;
                        }
                        if (checkEmptyValueLinks){
                            result = `${prefix}${value[0]}`;
                        }
                        if (likeAKikimor){
                            result = `<section class="fvtt advice"><figure class="icon"><img src="${dataBook.filePath[0].value}" class="round"></figure><article>${result}</article></section> `
                        }
                        let button = HtmlUtil.createElement("button", {elementClass: ["creatureChangeButton"], type:"button", name: "buttonTool"});
                        let img = HtmlUtil.createElement("img", {src: dataBook.filePath[0].value});
                        let label = HtmlUtil.createElement("label", {text: item});
                        HtmlUtil.appendElements(button, [img, label]);
                        button.setAttribute("onClick", `document.execCommand("insertHTML", false, '${result}')`)
                        dialogContent.appendChild(button);
                    }
                }
            }
            const myDialog = new Dialog({
                title: "Выбор",
                content: dialogContent.outerHTML,
                buttons: {}
            }).render(true);
        }) 

        html.on('click', '#macroSelectButton', () => {
            let dialogContent = HtmlUtil.createElement("div");
            dialogContent.appendChild(HtmlUtil.createElement("label", {text: `Выберите один из вариантов:\n`}));
            let selectedText;
            try {
                selectedText = this.getSelectedText();
            }catch {
                selectedText = '';
            }
            let macroSelect = html.find('[name="macroSelect"]').val();
            let EmbedValueLinks = html.find('[name="EmbedValueLinks"]')[0].checked;
            let checkEmptyValueLinks = html.find('[name="emptyValueLinks"]')[0].checked;
            let likeAKikimor = html.find('[name="likeAKikimor"]')[0].checked;
            let customName = html.find('[name="customName"]').val();
            let prefix = "@UUID"
            for (let items of macroSelect){
                for (let [item, value] of Object.entries(dataBook.macroSelect[items])){
                    if (item.toLowerCase().includes(selectedText.toLowerCase())){
                        if (EmbedValueLinks){
                            prefix = "@UUID"; //@EMBED
                        }
                        let result = `${prefix}${value[0]}{${selectedText}}`
                        if (customName){
                            result = `${prefix}${value[0]}{${customName}}`;
                        }
                        if (checkEmptyValueLinks){
                            result = `${prefix}${value[0]}`;
                        }
                        if (likeAKikimor){
                            result = `<section class="fvtt advice"><figure class="icon"><img src="${value[1]}" class="round"></figure><article>${result}</article></section> `
                        }
                        let button = HtmlUtil.createElement("button", {elementClass: ["creatureChangeButton"], type:"button", name: "buttonTool"});
                        let img = HtmlUtil.createElement("img", {src: value[1]});
                        let label = HtmlUtil.createElement("label", {text: item});
                        HtmlUtil.appendElements(button, [img, label]);
                        button.setAttribute("onClick", `document.execCommand("insertHTML", false, '${result}')`)
                        dialogContent.appendChild(button);
                    }
                }
            }
            const myDialog = new Dialog({
                title: "Выбор",
                content: dialogContent.outerHTML,
                buttons: {}
            }).render(true);
        }) 

        html.on('click', '#sceneButton', () => {
            let dialogContent = HtmlUtil.createElement("div");
            dialogContent.appendChild(HtmlUtil.createElement("label", {text: `Выберите один из вариантов:\n`}));
            let selectedText;
            try {
                selectedText = this.getSelectedText();
            }catch {
                selectedText = '';
            }
            let sceneSelect = html.find('[name="sceneSelect"]').val();
            let EmbedValueLinks = html.find('[name="EmbedValueLinks"]')[0].checked;
            let checkEmptyValueLinks = html.find('[name="emptyValueLinks"]')[0].checked;
            let likeAKikimor = html.find('[name="likeAKikimor"]')[0].checked;
            let customName = html.find('[name="customName"]').val();
            let prefix = "@UUID"
            for (let items of sceneSelect){
                for (let [item, value] of Object.entries(dataBook.sceneSelect[items])){
                    if (item.toLowerCase().includes(selectedText.toLowerCase())){
                        if (EmbedValueLinks){
                            prefix = "@UUID"; //@EMBED
                        }
                        let result = `${prefix}${value[0]}{${selectedText}}`
                        if (customName){
                            result = `${prefix}${value[0]}{${customName}}`;
                        }
                        if (checkEmptyValueLinks){
                            result = `${prefix}${value[0]}`;
                        }
                        if (likeAKikimor){
                            result = `<section class="fvtt advice"><figure class="icon"><img src="${value[1]}" class="round"></figure><article>${result}</article></section> `
                        }
                        let button = HtmlUtil.createElement("button", {elementClass: ["creatureChangeButton"], type:"button", name: "buttonTool"});
                        let img = HtmlUtil.createElement("img", {src: value[1]});
                        let label = HtmlUtil.createElement("label", {text: item});
                        HtmlUtil.appendElements(button, [img, label]);
                        button.setAttribute("onClick", `document.execCommand("insertHTML", false, '${result}')`)
                        dialogContent.appendChild(button);
                    } //entity.background.src
                }
            }
            const myDialog = new Dialog({
                title: "Выбор",
                content: dialogContent.outerHTML,
                buttons: {}
            }).render(true);
        }) 

        html.on('click', '#buttonAwords', () => {
            let pp = html.find('[id="pp"]')[0].value;
            let gp = html.find('[id="gp"]')[0].value;
            let ep = html.find('[id="ep"]')[0].value;
            let sp = html.find('[id="sp"]')[0].value;
            let cp = html.find('[id="cp"]')[0].value;
            let xp = html.find('[id="xp"]')[0].value;
            let partyCheck = html.find('[id="party"]');
            let eachCheck = html.find('[id="each"]');
            let party = '';
            let each = '';
            if (partyCheck[0].checked) {
                party = "party ";
            }
            if (eachCheck[0].checked) {
                each = "each ";
            }
            if (pp){pp = `${pp}pp `;} else {pp = ``;}
            if (gp){gp = `${gp}gp `;} else {gp = ``;}
            if (ep){ep = `${ep}ep `;} else {ep = ``;}
            if (sp){sp = `${sp}sp `;} else {sp = ``;}
            if (cp){cp = `${cp}cp `;} else {cp = ``;}
            if (xp){xp = `${xp}xp `;} else {xp = ``;}
            let result = `${party}${pp}${gp}${ep}${sp}${cp}${xp}${each}`
            result = result.trim();
            if (result !== '' && result !== 'party' && result !== 'each' && result !== 'party each' ){
                let newContent = `[[/award ${result}]]`;
                document.execCommand('insertHTML', false, newContent);
            }
        })

        html.on('click', '#buttonClear', () => {
            html.find('[id="pp"]')[0].value = '';
            html.find('[id="gp"]')[0].value = '';
            html.find('[id="ep"]')[0].value = '';
            html.find('[id="sp"]')[0].value = '';
            html.find('[id="cp"]')[0].value = '';
            html.find('[id="xp"]')[0].value = '';
            html.find('[id="party"]').prop( "checked", false );
            html.find('[id="each"]').prop( "checked", false );
        })

        html.on('click', '#narrative', () => {
            let selectedText = this.getSelectedText();
            let newContent = `
            <section class="fvtt-relative">
                <section class="fvtt narrative">
                    ${selectedText}
                </section>
            </section>
            `;
            document.execCommand('insertHTML', false, newContent);
        })

        html.on('click', '#advice', () => {
            let selectedText = this.getSelectedText();
            let circle = '';
            if (html.find('[name="onCircle"]')[0].checked) {
                circle = 'class="round"';
            }
            let icon = html.find('[name="picturesInput"]')[0].value;
            let newContent = `
            <section class="fvtt advice">
                <figure class="icon">
                    <img src=${icon} ${circle}>
                </figure>
                <article>
                    ${selectedText}
                </article>
            </section>
            `;
            document.execCommand('insertHTML', false, newContent);
        })

        html.on('click', '#notable', () => {
            let selectedText = this.getSelectedText();
            let newContent = `
            <aside class="notable">
                ${selectedText}
            </aside>

            `;
            document.execCommand('insertHTML', false, newContent);
        })

        html.on('click', '#buttonColorAdd', async () => {
            await game.user.setFlag(moduleName, "userColors", null);
            let color = dataBook.colorViget[0].value;
            let name = html.find('[id="nameColor"]')[0].value;
            dataBook.userColors[color] = name;
            await dataBook.select.find('option').remove();
            await game.user.setFlag(moduleName, "userColors", dataBook.userColors);
            for (const [_color, _name] of Object.entries(dataBook.userColors)) {
                await dataBook.select.append(`<option style="background-color: ${_color};" value=${_color} >${_name}</option>`);
            }
            dataBook.select.val(color).selected = true;
            await dataBook.select.trigger('change');
        })

        html.on('click', '#buttonColor', async () => {
            let color = dataBook.color;
            let selectedText = document.getSelection().toString();
            let newContent = `<span style="color:${color};">${selectedText}</span>`;
            document.execCommand('insertHTML', false, newContent);
        })

        html.on('click', '#buttonColorBackground', async () => {
            let color = dataBook.color;
            let selectedText = this.getSelectedText();
            let newContent = `<div style="padding:0px 5px;background-color:${color};border:1px outset black;box-shadow:2px 2px 2px;border-radius:5px">${selectedText}</div>`;
            document.execCommand('insertHTML', false, newContent);
         
        })

        html.on('click', '#buttonDeleteColor', async () => {
            await game.user.setFlag(moduleName, "userColors", null);
            await delete dataBook.userColors[dataBook.select[0].value];
            await dataBook.select.find('option').remove();
            await game.user.setFlag(moduleName, "userColors", dataBook.userColors);
            for (const [_color, _name] of Object.entries(dataBook.userColors)) {
                await dataBook.select.append(`<option style="background-color: ${_color};" value=${_color} >${_name}</option>`);
            }
            dataBook.select.trigger('change');
        })

        html.on('click', '#buttonColorBorder', async () => {
            let selectedText = this.getSelectedText();
            let style = html.find('[name="boerderSelect"]')[0].value;
            let size = html.find('[id="borderSize"]')[0].value;
            let unit = html.find('[id="boerderUnitSelect"]')[0].value;
            let angle = html.find('[id="angle"]')[0].value;
            let result = `<div style="padding:0px; background-color: ${dataBook.backgroundColorBorder}; border-radius: ${angle}%; border-color: ${dataBook.color}; border-width:${size}${unit}; border-style: ${style};">${selectedText}</div>`;
            document.execCommand('insertHTML', false, result);
        })

        html.on('click', '#buttonBackgroundColorBorder', async () => {
            dataBook.backgroundColorBorder = dataBook.color;
            html.find('[name="boerderSelect"]').trigger('change');
        })

        html.find('.itemRef').click((e) => {
            const clickedTabId = e.currentTarget.dataset.tab;
            if (clickedTabId == "tab1" | clickedTabId == "tab2" | clickedTabId == "tab6"){
                html.find('.numberCheck').show();
                html.find('.skillPassiveCheck').show();
                html.find('.numberForMod').show();
            }
            else{
                html.find('.numberCheck').hide();
                html.find('.skillPassiveCheck').hide();
                html.find('.numberForMod').hide();
            }
            if (clickedTabId == "tab5"){
                html.find('.damageFormula').show();
                html.find('.numberCheck').show();
            }
            else{
                html.find('.damageFormula').hide();
            }
        });

        html.on('change', '#transparent', (event) => {
            let transparent = html.find('[id="transparent"]').val();
            let color = dataBook.colorViget[0].value;
            var r = parseInt(color.slice(1, 3), 16),
                g = parseInt(color.slice(3, 5), 16),
                b = parseInt(color.slice(5, 7), 16);
            dataBook.color = "rgba(" + r + ", " + g + ", " + b + ", " + transparent + ")";
            dataBook.colorHex[0].value = color
            html.find('[name="boerderSelect"]').trigger('change');
        })  
        
        html.on('change', '#colour', (event) => {
            let transparent = html.find('[id="transparent"]').val();
            let color = dataBook.colorViget[0].value;
            var r = parseInt(color.slice(1, 3), 16),
                g = parseInt(color.slice(3, 5), 16),
                b = parseInt(color.slice(5, 7), 16);
            dataBook.color = "rgba(" + r + ", " + g + ", " + b + ", " + transparent + ")";
            dataBook.colorHex[0].value = color;
            html.find('[name="boerderSelect"]').trigger('change');
        })

        html.on('change', '#angle', (event) => {
            html.find('[name="boerderSelect"]').trigger('change');
        })

        html.on('change', '.boerderSelect', (event) => {
            let style = html.find('[name="boerderSelect"]')[0].value;
            let template = html.find('[name="borderTemplate"]');
            let size = html.find('[id="borderSize"]')[0].value;
            let unit = html.find('[id="boerderUnitSelect"]')[0].value;
            let angle = html.find('[id="angle"]')[0].value;
            template.css({
                "background-color": `${dataBook.backgroundColorBorder}`,
                "border-radius": `${angle}%`,
                "border-color": dataBook.color, 
                "border-width":`${size}${unit}`, 
                "border-style":style,
                "margin": "auto",
                "width": "70%"
            }); 
        })

        html.on('change', '#boerderUnitSelect', (event) => {
            html.find('[name="boerderSelect"]').trigger('change');
        })

        html.on('change', '#borderSize', (event) => {
            html.find('[name="boerderSelect"]').trigger('change');
        })
        
        html.on('change', '#colorHex', (event) => {
            let transparent = html.find('[id="transparent"]').val();
            let color = dataBook.colorHex[0].value
            if (color.startsWith('#')){
                dataBook.colorViget[0].value = dataBook.colorHex[0].value;
                let color = dataBook.colorViget[0].value;
                var r = parseInt(color.slice(1, 3), 16),
                    g = parseInt(color.slice(3, 5), 16),
                    b = parseInt(color.slice(5, 7), 16);
                dataBook.color = "rgba(" + r + ", " + g + ", " + b + ", " + transparent + ")";
                dataBook.colorHex[0].value = color
            }
            html.find('[name="boerderSelect"]').trigger('change');
        })

        html.on('change', '#numberCheck', (event) => {
            if (event.currentTarget.checked) {
                html.find('[id="numberForMod"]').prop( "disabled", true);
                html.find('[id="damageFormula"]').prop( "disabled", true);
              } else {
                html.find('[id="numberForMod"]').prop( "disabled", false);
                html.find('[id="damageFormula"]').prop( "disabled", false);
              }
        })

        html.on('change', '#onCircle', (event) => {
            if (event.currentTarget.checked) {
                dataBook.fileImg.addClass(['round']);
                dataBook.buttonImg.addClass(['round']);
                game.user.setFlag(moduleName, "rounded", true)
              } else {
                dataBook.fileImg.removeClass(['round']);
                dataBook.buttonImg.removeClass(['round']);
                game.user.setFlag(moduleName, "rounded", false)
              }
        })

        html.on('change', '#userPalette', (event) => {
            dataBook.colorViget[0].value = dataBook.select[0].value;
            dataBook.select.css("background", dataBook.select[0].value);
        })

        html.on('change', '#leftCheck', (event) => {
            if (event.currentTarget.checked) {
                html.find('[name="rightCheck"]').prop( "checked", false );
              }
        })

        html.on('change', '#rightCheck', (event) => {
            if (event.currentTarget.checked) {
                html.find('[name="leftCheck"]').prop( "checked", false );
            }
        })


        html.find('#refImg').on('dragover', (event) => event.preventDefault());
        html.find('#refImg').on('drop', async (event) => {
            const entityData = JSON.parse(event.originalEvent.dataTransfer.getData("text/plain"));
            const entity = await fromUuid(entityData.uuid);
            if (entity.uuid.includes("Actor")) {
                await this.picturies_dialog(entity)
            }else{
                this.make_path(entity.img || entity.background.src);
            }
        });

        
    }

    async picturies_dialog(actor){
        // Определим пути к изображениям
        const imagePath1 = actor.img; // Замените на свой путь
        const imagePath2 = actor.prototypeToken.texture.src; // Замените на свой путь
    
        await new Dialog({
            title: "Выберите изображение",
            content: ``,
            buttons: {
                left: {
                    label: `<img src="${imagePath1}" style="width: auto; height: 100px;" alt="Image 1">`,
                    callback: () => {
                        this.make_path(imagePath1);
                    }
                  },
                  
                right: {
                label: `<img src="${imagePath2}" style="width: auto; height: 100px;" alt="Image 2">`,
                callback: () => {
                    this.make_path(imagePath2);
                }
                },
            },
            default: "",
            close: () => {},
        }).render(true);
    }
}

